########################################################################################################################
#### Successione Ecologica
####
#### INTRODUZIONE ALLANALISI MULTIVARIATA CON R
#### #iorestoacasa #acasaconse
####
#### Marco Sigovini
#### CNR-ISMAR - Venezia
#### marco.sigovini@ve.ismar.cnr.it
####
#### Il presente documento  rilasciato con licenza CC BY 4.0 (https://creativecommons.org/licenses/by/4.0/deed.it)
####
########################################################################################################################
#### 1) AVVIO R

## installazione pacchetti (cluster, analisi multivariate in ecologia) da repository
install.packages(c("rgl", "cluster", "vegan", "labdsv", "psych", "dendextend", "ade4"), repos = "https://cran.stat.unipd.it/")


## caricamento pacchetti
library(cluster)    # cluster analysis
library(vegan)      # community ecology
library(labdsv)     # enhance vegan
library(psych)      # some tool
library(rgl)        # 3D plots
#library(dendextend) # functions for extending 'dendrogram' objects    (NON UTILIZZATO NEL PRESENTE TUTORIAL)
#library(ade4)       # alternative comprehensive package to vegan      (NON UTILIZZATO NEL PRESENTE TUTORIAL)


## impostazione cartella di lavoro (INSERIRE PERCORSO PERSONALIZZATO)
#setwd("C:/Users/Marco")


## caricamento dataset di esempio disponibili nei pacchetti
data(mite)
data(mite.env)
data(mite.xy)

head(mite)

## esempio di export e di caricamento dataset personale
#write.csv(mite, "dataset.csv")                              # esempio di export come csv
#esempio <- read.csv("dataset.csv", row.names = 1)           # esempio di import csv



########################################################################################################################
#### 2) ESPLORAZIONE E PREPARAZIONE DATASET

## dati di comunit (dataset "Oribatid mites": acari oribatidi)
str(mite)
class(mite)      # classe "data.frame"
summary(mite)
range(mite)

## trasformazione dati di comunit: trasformazione moderata (radice quadrata) per dare pi peso a specie rare
mite.tr <- sqrt(mite)
range(mite.tr)


## variabili ambientali (riferite al dataset "Oribatid mites")
str(mite.env)
head(mite.env)
summary(mite.env)

## esplorazione grafica variabili
pairs(mite.env[, 1:2])
pairs.panels(mite.env[, 1:2], smooth = F, ellipses = F)    # funzione in pacchetto "psych"


## standardizzazione variabili quantitative
mite.env.st <- mite.env
mite.env.st[, c(1:2)] <- decostand(mite.env.st[, c(1:2)], MARGIN = 2, "standardize")         # standardizzazione

head(as.matrix(mite.env))                                  # differenza tra 'data.frame' e 'matrix' in R
head(as.matrix(mite))


## variabili spaziali (coordinate) e distribuzione delle stazioni (dataset "Oribatid mites")
head(mite.xy)
plot(mite.xy, asp = 1)


#################################################################################################################
#### 3) CLASSIFICAZIONE: HIERARCHICAL AGGLOMERATIVE CLUSTER ANALYSIS

## matrice dissimilarit
mite.dis <- vegdist(mite.tr, method = "bray")          # fuzione vegdist: indice di Bray-Curtis default
                                                       # altri indici: "euclidean", "jaccard", "gower", etc
class(mite.dis)
str(mite.dis)                         # di default esclude la parte triangolare superiore e la diagonale
range(mite.dis)
vegdist(mite.tr, diag = T)            # dissimilarit di campione con se stesso  pari a zero


## single linkage: la similarit tra i due cluster  pari a quella tra i membri pi simili
mite.sl <- hclust(mite.dis, method = "single")
class(mite.sl)
str(mite.sl)

## plot cluster
plot(mite.sl, hang = -1)              # funzione plot classe "hclust"
par(mfrow = c(3, 1))                  # impostazione parametri per plottare tre grafici in colonna
plot(as.dendrogram(mite.sl), main = "single linkage")   # funzione plot classe "dendrogram"
as.dendrogram(mite.sl)                # classe 'dendrogram'
str(as.dendrogram(mite.sl))

## average linkage: la similarit tra i due cluster  pari a quella media
mite.al <- hclust(mite.dis, method = "average")
plot(as.dendrogram(mite.al), main = "average linkage")


## complete linkage: la similarit tra i due cluster  pari a quella tra i membri meno simili
mite.cl <- hclust(mite.dis, method = "complete")
plot(as.dendrogram(mite.cl), main = "complete linkage")

par(mfrow = c(1, 1))                  # ritorno a impostazioni grafiche normali


## rappresentazione distribuzione specie tra cluster
tabasco(mite.tr, mite.al, scale = "Hill")


## identificazione cluster
plot(as.dendrogram(mite.al), main = "average linkage")
abline(h = 0.5, col = "red")
cutree(mite.al, k = 6)                 # in base al numero di cluster voluti
cutree(mite.al, h = 0.5)               # in base alla dissimilarit


## modifica simbologia cluster tramite utilizzo funzione (esempio: in base a variabile qualitativa "Topo")

key <- data.frame("Topo" = c(levels(mite.env$Topo)), "col" = c("red", "blue"))
key
Topo.col <- as.character(key$col[match(mite.env$Topo, key$Topo)])

i = 0                                 # set iniziale
colLab <- function(n){                # funzione (sintassi avanzata!)
           if(is.leaf(n)){
            a <- attributes(n)
            col <- Topo.col[as.integer(a$label)]
            attr(n, "nodePar") <- c(attributes(n)$nodePar, list(lab.col = col, pch = NA))
            }
           return(n)
	     }                          # fine funzione

plot(dendrapply(as.dendrogram(mite.al), colLab))


#### SINTESI
plot(as.dendrogram(hclust(vegdist(sqrt(mite)), "average")))


#################################################################################################################
#### 4) ORDINAMENTO: NONMETRIC MULTIDIMENSIONAL SCALING (NMDS)
#### metodo basato su matrice dissimilarit, non metrico
#### algoritmo iterativo, parte da configurazione random e cerca di minimizzare funzione di "stress"
#### calcolata attraverso il confronto tra la configurazione ottenuta (distanze metriche nell'ordinamento) con la 
#### dissimilarit originale (in genere ricercato stress inferiore a 0.2 o 0.3)
#### soluzioni potrebbero non convergere, sono possibili diversi "local optima"
#### esistono diverse implementazioni dell'algoritmo (diversi criteri di ottimizzazione, etc.)
#### specie possono essere rappresentate ex-post come weighted averages
#### versione metrica: PCoA (Metric Multidimensional Scaling)


## funzione 'metaMDS': non esegue direttamente NMDS: "wrapper" che racchiude i vari passaggi
mite.mds <- metaMDS(mite.tr, distance = "bray",   # matrice dei dati e indice di dissimilarit (come da da 'vegdist')
                                                  # (in alternativa, matrice di dissimilarit)
               k = 2,                             # numero di dimensioni
               try = 20, trymax = 20,             # min e max iterazioni per ricerca soluzione stabile (valori di default)
               engine = "monoMDS",                # funzione che esegue NMDS (default: 'monoMDS' di vegan)
               autotransform = F,                 # procedura automatica di trasformazione dei dati? (default: T)
               wascores = T)                      # calcolo scores relativi alle specie (weighted averages)? (default: T)

mite.mds
class(mite.mds)
str(mite.mds)


## grafico di Shepard (dissimilarit vs distanze NMDS)
stressplot(mite.mds)


## plot NMDS
plot(mite.mds, type = "text", display = "sites")             # funzione plot classe "metaMDS"


## confronto con e senza "autotrasform"
par(mfrow = c(1, 2))
plot(metaMDS(mite.tr, autotransform = F), display = "sites")
plot(metaMDS(mite, autotransform = T), display = "sites")    # sulla base delle caratteristiche dataset esegue sqrt e Wisconsin
                                                             # (specie standardizzate sui max, siti sul totale)
par(mfrow = c(1, 1))


## ancora plot NMDS
ordiplot(mite.mds, type = "text", display = "sites")         # funzione plot ordinamenti generica pacchetto 'vegan'

pl <- ordiplot(mite.mds, type = "none")                      # 'type = "none"' permette maggior controllo parametri
points(pl, "sites")
identify(pl, "sites")                                        # permette identificazione punti interattiva

pl <- ordiplot(mite.mds, type = "none")
text(pl, "sites", col = cutree(mite.al, k = 6))              # nomi stazioni colorati (con colori casuali)
ordihull(mite.mds, groups = cutree(mite.al, k = 6))          # convex hull basato su fattori (ad es. cluster)

ordiplot(mite.mds, type = "text", display = "sites")
ordiellipse(mite.mds, groups = cutree(mite.al, k = 6), kind = "sd")    # centroidi e dispersione (sd)


#################################################################################################################
#### 5) ORDINAMENTO: METODI BASATI SUGLI AUTOVETTORI, "UNCONSTRAINED"
#### metodi non parametrici basati sull'analisi degli autovettori della matrice originale (PCA), della matrice 
#### (CA) o di una generica matrice di dissimilarit (PCoA)
#### l'inerzia  una misura della varianza totale del dataset ed  decomposta (come autovalori) negli assi dell'ordinamento
#### nel caso di PCA e CA, l'output principale  un grafico 2D del tipo "biplot", 
#### in cui i due assi sono in genere quelli che spiegano la maggior parte della varianza, ed in cui sono 
#### simultaneamente rappresentati (con fattore di scala adeguato) le nuove coordinate (scores) degli oggetti (campioni)
#### e delle variabili di interesse


## caricamento e preparazione dataset 3D (variabili ambientali)
data(varechem)
ex <- varechem[c(1, 2, 14)]
head(ex)
ex.st <- decostand(ex, method = "standardize")


## PRINCIPAL COMPONENT ANALYSIS (PCA)
## metodo "constrained" corrispondente: Redundancy Analysis (RDA)
## applicato a matrice di variabili reali, con medesima udm o standardizzate (in genere dati ambientali)
## consiste nella semplice rotazione della nuvola di dati ("preserva la distanza euclidea")
## performa meglio quando dati presentano distribuzione monotona lungo gradiente
## interpretazione pi semplici quando distribuzione multivariata normale (ma non  assunzione!)
## l'inerzia ha un significato diretto:  uguale alla varianza totale della matrice delle abbondanze

ex.st.pca <- rda(ex.st)                 # la funzione 'rda' include PCA e RDA
class(ex.st.pca)
ex.st.pca                               # nella PCA "Inertia is variance" 
sum(apply(ex.st, 2, var))               # varianza totale matrice
str(ex.st.pca)
ex.st.pca$CA$eig
eigenvals(ex.st.pca)                    # autovalori
sum(eigenvals(ex.st.pca))               # inerzia
scores(ex.st.pca)                       # sites scores
summary(ex.st.pca)


## biplot
biplot(ex.st.pca)                       # funzione biplot generica, utile per plot rapido
plot(ex.st.pca)                         # funzione biplot classe "cca" "rda"
ordiplot(ex.st.pca)                     # funzione plot ordinamenti generica pacchetto 'vegan'
ordiplot(ex.st.pca, type = "text")


## esempio di plot con controllo vari elementi
pl <- ordiplot(ex.st.pca, type = "none", scaling = 2)
text(pl, "sites", col = "red", pch = 16)
text(pl, "species", arrows = T, col = "blue")            # variabili rappresentate con frecce

## NB:
## 'scaling = 1': "distance biplot", appropriato per rappresentare la posizione reciproca dei campioni
## 'scaling = 2': "correlation biplot", appropriato per rappresentare la relazione angolare tra le variabili (default)


## plot 3D: comprensione ordinamento PCA (package "rgl")
par3d(FOV = 0)
plot3d(ex.st, type = "n")
text3d(ex.st, texts = rownames(ex.st))        # si pu ruotare plot 3d in modo interattivo


## CORRESPONDENCE ANALYSIS (CA)
## metodo "constrained" corrispondente: Canonical CA (CCA)
## applicato a "matrice di contingenza": dati non negativi, stessa udm, comparabili a "frequenze" (dati di comunit)
## preserva la distanza chi-quadro tra i campioni, cos come quella tra le specie (non soffre del "double-zero problem")
## performa meglio quando dati presentano distribuzione unimodale lungo gradiente
## (spesso presenta nel biplot un artefatto ad "arco")

mite.tr.ca <- cca(mite.tr)                    # la funzione 'cca' include CA e CCA
class(mite.tr.ca)
mite.tr.ca

par(mfrow = c(1, 2))
plot(mite.tr.ca, scaling = 1, main = "CA, scaling = 'sites'")
plot(mite.tr.ca, scaling = 2, main = "CA, scaling = 'species'")
par(mfrow = c(1, 1))                          # variabili rappresentate come centroidi

## NB:
## 'scaling = 1' ('sites'): i campioni sono medie pesate delle specie; preserva la distanza chi-quadro tra i campioni
##    appropriato per rappresentare la posizione relativa dei campioni (il contributo delle specie  indicato dalla distanza dal campione)
## 'scaling = 2' ('species'): le specie sono medie pesate dei campioni; preservata la distanza chi-quadro tra le specie
##    appropriato per rappresentare la posizione relativa delle specie (la frequenza nei campioni  indicata dalla distanza dalla specie)
## 'scaling = 3' ('symmetric'): rappresentazione di compromesso, non  preservata la distanza chi-quadro n per specie n per campioni


## TRANSFORMATION-BASED PCA
## consente di applicare PCA a matrici di comunit previa trasformazione, in questo esempio: trasf. di Hellinger
## trasformazione di Hellinger + distanza Euclidea => distanza di Hellinger (appropriata per i dati di comunit)
## l'inerzia ha un significato diretto:  uguale alla varianza totale della matrice delle abbondanze

mite.hel <- decostand(mite.tr, "hellinger")
mite.hel.pca <- rda(mite.hel) 
mite.hel.pca

par(mfrow = c(1, 2))
plot(mite.tr.ca, scaling = 2, main = "CA")
plot(mite.hel.pca, main = "tbPCA")
par(mfrow = c(1, 1))


#################################################################################################################
#### 6) ORDINAMENTO: METODI BASATI SUGLI AUTOVETTORI, "CONSTRAINED"
#### estensione dei rispettivi metodi non parametrici "unconstrained" in cui analizzate contemporaneamente 
#### un dataset di variabili di interesse (ad es. dati di comunit) ed uno di predittori (ad es. variabili ambientali)
#### estratti assi "constrained", corrispondenti alla direzione di variabilit dei dati spiegata dai predittori
#### l'inerzia  una misura della varianza totale del dataset ed  decomposta (come autovalori) negli assi 
#### "contrained" (pari al numero di variabili ambientali) e "unconstrained" dell'ordinamento
#### nel caso di RDA e CCA, l'output principale  un grafico 2D del tipo "triplot", in cui rispetto al biplot 
#### sono rappresentate anche i predittori


## TRANSFORMATION-BASED REDUNDANCY ANALYSIS (tbRDA)
## consente di applicare PCA a matrici di comunit previa trasformazione, in questo esempio: trasf. di Hellinger
## trasformazione di Hellinger + distanza Euclidea => distanza di Hellinger (appropriata per i dati di comunit)

summary(mite.env.st)

mite.hel.rda <- rda(mite.hel ~ SubsDens + WatrCont, data = mite.env.st)  # sintassi del tipo "formula" (~ : ALT+126)
mite.hel.rda
plot(mite.hel.rda)                 # 'display = "wa"': rappresentazione default di "site scores"

## NB:
## in metodi "constrained", rappresentazione alternative di "site scores":
## come medie pesate di "species scores" ("wa") (espressione comunit, pi robuste a errori random nelle var. ambientali)
## combinazioni lineari delle variabili esplicative ("lc") (espressione ambiente)



mite.hel.rda2 <- rda(mite.hel ~ ., data = mite.env.st)           # nella sintassi "formula", '.' indica tutte le variabili
mite.hel.rda2                                                    # il numero di assi vincolati  pari a numero var. quantitative
                                                                 # pi numero fattori (- 1) per ogni var. categorica
plot(mite.hel.rda2)                                              # fattori var. categoriche rappresentati come centroidi
plot(mite.hel.rda2, xlim = c(-0.5, 0.5), ylim = c(-0.5, 0.5))    # ingrandimento



#################################################################################################################
#### 7) TEST INFERENZIALI
#### in genere non parametrici, pasati su permutazioni
#### calcolata statistica (ad es. 'R') e p-value (probabilit di ottenere un valore della statistica pari 
#### (o pi estremo) di quello osservato, assumendo che H0 (ipotesi nulla)  vera)
#### -> p-value minore di alpha: si rigetta H0: evidenza statisticamente significativa per inferire che ipotesi alternativa sia vera
#### -> p-value maggiore di alpha: H0 non si pu rigettare: manca evidenza per inferire che ipotesi alternativa sia vera


## ANALYSIS OF SIMILARITIES (ANOSIM)
## test statistico non parametrico per valutare differenza significativa tra due o pi gruppi di campioni
## dissimilarit nella composizione tra gruppi deve essere maggiore rispetto a quella all'interno dei gruppi
## basato su ranghi dissimilarit (concettualmente analogo a NMDS, a cui  spesso accompagnato)
## produce una statistica R (-1, 1) basata sulla differenza tra media dei ranghi tra gruppi e entro gruppi (mai valori <<0)
## la significativit di R  valutata tramite permutazione dei gruppi (per produrre distribuzione empirica di R sotto H0)


pl <- ordiplot(mite.mds, type = "none")
text(pl, "sites", col = Topo.col)                                # nomi stazioni colorati in base a gruppi a priori
ordihull(mite.mds, groups = mite.env.st$Topo, lty = 3)           # convex hull basato su fattori (ad es. cluster)
ordiellipse(mite.mds, groups = mite.env.st$Topo, kind = "sd")    # centroidi e dispersione (sd)


mite.anosim <- anosim(mite.dis, mite.env.st$Topo, permutations = 999)   # ANOSIM 1-way, 2 gruppi
mite.anosim
summary(mite.anosim)
plot(mite.anosim)                  # boxplot


## PERMANOVA
## test statistico non parametrico per valutare differenza significativa tra due o pi gruppi di campioni
## versione permutazionale di Multivariate ANOVA (MANOVA), partiziona la matrice delle dissimilarit tra fonti di variazione 
## (concettualmente analogo a dbRDA e, con distanze euclidee, a RDA)
## produce pseudo-F ratio (al posto di F ratio di Fisher) e R^2
## la significativit di R  valutata tramite permutazione dei gruppi (per produrre distribuzione empirica di R sotto H0)
## disegni sperimentali anche complessi (fattori multipli crossed e nested, mixed models), approccio pi robusto e versatile di ANOSIM


mite.adonis <- adonis(mite.dis ~ mite.env.st$Topo, data = mite.env.st, permutations = 999)   # PERMANOVA 1-way, 2 gruppi
mite.adonis                 # classica tabella anova: fonti di variazione, gdl, SS, MS, F, R^2, p-val (permutazionale)





#################################################################################################################
#################################################################################################################
#################################################################################################################
#################################################################################################################
#### SITI E TUTORIAL CONSIGLIATI
#### 
#### http://biol09.biol.umontreal.ca/Trieste16/
#### http://adn.biol.umontreal.ca/~numericalecology/Trieste16/
#### http://ordination.okstate.edu/
#### https://www.davidzeleny.net/anadat-r/doku.php/en:start
#### http://ecology.msu.montana.edu/labdsv/R/labs/
#### https://pbil.univ-lyon1.fr/ade4/home.php?lang=eng
#### http://dmcglinn.github.io/quant_methods/lessons/multivariate_models.html
#### https://www.flutterbys.com.au/stats/index.html



#### v. 16/04/2020 ##############################################################################################

